//
// Project Name: Bouncing Box Animation
//
// Author: Matthew Weathers
// Date  : 26-October-2000, Modified 5-November-2000

#include <cstdlib>
#include <cstdio>
#include <cmath>
#include <ctime>
#include <cstring>
#include "graphics2.h"

#define ESC 0x1b
#define MAX_X  700
#define MAX_Y  500

void DrawBox(int, int, int);

void main() 
{
	srand(time(NULL));  // Seed the random number generator
	int GraphDriver=0,GraphMode=0;
	initgraph( &GraphDriver, &GraphMode, "", MAX_X, MAX_Y ); // Start Window
	char tempstring[80];

	int mouseX, mouseY;

	float x_pos;
	float y_pos;
	float x_vel;
	float y_vel;
	float x_prev;
	float y_prev;
	int color;

	bool KeepGoing, gravity_on, change_color_on, trails_on;
	char KeyPressed;

	x_pos = MAX_X / 2.0;   // Center of screen
	y_pos = MAX_Y / 2.0;
	x_prev = x_pos;
	y_prev = y_pos;
	x_vel = 6.2;
	y_vel = 3.4;
	color=YELLOW;
	gravity_on=false;
	change_color_on=false;
	trails_on=false;

	setcolor(WHITE);
	settextstyle(TRIPLEX_FONT, HORIZ_DIR, 1);
	outtextxy(20, 20, "Press ESC to quit, click to change position");
	outtextxy(20, 40, "G-Gravity on/off C-colors on/off T-Trails on/off");

	KeepGoing = true;
	while (KeepGoing) {

		// Update positions
		x_pos += x_vel;
		y_pos += y_vel;

		if (change_color_on) {
			color = (color+1)%64;
		}

		//Gravity
		if (gravity_on) {
			y_vel += 0.4;
		}

		// Erase box from previous position
		if (! trails_on) {
			DrawBox(x_prev, y_prev, BLACK);
		}

		// Draw box in new position
		DrawBox(x_pos, y_pos, color);

		// Remember previous position
		x_prev = x_pos;
		y_prev = y_pos;

		// Bounce off left & right wall
		if ((x_pos<20) || (x_pos>MAX_X-20)) {
			x_vel *= -1.0;
		}

		// Bounce off floor & ceiling
		if ((y_pos<20) || (y_pos>MAX_Y-20)) {
			y_vel *= -1.0;
		}

		// Check keyboard
		if (kbhit()) {
			KeyPressed=getch();

			switch(KeyPressed) {
			case ESC: case 'q': case 'Q':
				//Quit
				KeepGoing = false;
				break;
			case 'F': case 'f':
				// Go faster
				x_vel *= 2.0;
				y_vel *= 2.0;
				break;
			case 's': case 'S':
				// Go slower
				x_vel *= 0.5;
				y_vel *= 0.5;
				break;
			case 'g': case 'G':
				// Toggle gravity
				if (gravity_on) {
					gravity_on = false;
				} else {
					gravity_on = true;
				}
				break;
			case 't': case 'T':
				trails_on = trails_on?false:true; //Toggle (shortcut)
				break;
			case 'c': case 'C':
				change_color_on = change_color_on?false:true; //Toggle (shortcut)
				break;
			default:
				// Unrecognized key, do nothing
				break;
			}
		} // end of if kbhit

		// Check mouse
		if ( mouseup() ) {
			mouseX = mouseclickx();
			mouseY = mouseclicky();

			x_pos = mouseX;
			y_pos = mouseY;
		} // end of if mouseup

		delay(50);
	} // end of while KeepGoing

	//getch(); //Wait for a key. (When main function ends, the window will close)
} // end of main()

void DrawBox(int x, int y, int color){
	setcolor(color);
	setfillstyle(SOLID_FILL, color);
	bar(x-10,y-10,x+10,y+10);
}
